// Wild Magic Source Code
// David Eberly
// http://www.geometrictools.com
// Copyright (c) 1998-2007
//
// This library is free software; you can redistribute it and/or modify it
// under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation; either version 2.1 of the License, or (at
// your option) any later version.  The license is available for reading at
// either of the locations:
//     http://www.gnu.org/copyleft/lgpl.html
//     http://www.geometrictools.com/License/WildMagicLicense.pdf
// The license applies to versions 0 through 4 of Wild Magic.
//
// Version: 4.0.0 (2006/06/28)

#include "Wm4FoundationPCH.h"
#include "Wm4DistVector2Box2.h"

namespace Wm4
{
//----------------------------------------------------------------------------
template <class Real>
DistVector2Box2<Real>::DistVector2Box2 (const Vector2<Real>& rkVector,
    const Box2<Real>& rkBox)
    :
    m_rkVector(rkVector),
    m_rkBox(rkBox)
{
}
//----------------------------------------------------------------------------
template <class Real>
const Vector2<Real>& DistVector2Box2<Real>::GetVector () const
{
    return m_rkVector;
}
//----------------------------------------------------------------------------
template <class Real>
const Box2<Real>& DistVector2Box2<Real>::GetBox () const
{
    return m_rkBox;
}
//----------------------------------------------------------------------------
template <class Real>
Real DistVector2Box2<Real>::Get ()
{
    return Math<Real>::Sqrt(GetSquared());
}
//----------------------------------------------------------------------------
template <class Real>
Real DistVector2Box2<Real>::GetSquared ()
{
    // work in the box's coordinate system
    Vector2<Real> kDiff = m_rkVector - m_rkBox.Center;

    // compute squared distance and closest point on box
    Real fSqrDistance = (Real)0.0, fDelta;
    Vector2<Real> kClosest;
    int i;
    for (i = 0; i < 2; i++)
    {
        kClosest[i] = kDiff.Dot(m_rkBox.Axis[i]);
        if (kClosest[i] < -m_rkBox.Extent[i])
        {
            fDelta = kClosest[i] + m_rkBox.Extent[i];
            fSqrDistance += fDelta*fDelta;
            kClosest[i] = -m_rkBox.Extent[i];
        }
        else if (kClosest[i] > m_rkBox.Extent[i])
        {
            fDelta = kClosest[i] - m_rkBox.Extent[i];
            fSqrDistance += fDelta*fDelta;
            kClosest[i] = m_rkBox.Extent[i];
        }
    }

    m_kClosestPoint0 = m_rkVector;
    m_kClosestPoint1 = m_rkBox.Center;
    for (i = 0; i < 2; i++)
    {
        m_kClosestPoint1 += kClosest[i]*m_rkBox.Axis[i];
    }

    return fSqrDistance;
}
//----------------------------------------------------------------------------
template <class Real>
Real DistVector2Box2<Real>::Get (Real fT, const Vector2<Real>& rkVelocity0,
    const Vector2<Real>& rkVelocity1)
{
    Vector2<Real> kMVector = m_rkVector + fT*rkVelocity0;
    Vector2<Real> kMCenter = m_rkBox.Center + fT*rkVelocity1;
    Box2<Real> kMBox(kMCenter,m_rkBox.Axis,m_rkBox.Extent);
    return DistVector2Box2<Real>(kMVector,kMBox).Get();
}
//----------------------------------------------------------------------------
template <class Real>
Real DistVector2Box2<Real>::GetSquared (Real fT,
    const Vector2<Real>& rkVelocity0, const Vector2<Real>& rkVelocity1)
{
    Vector2<Real> kMVector = m_rkVector + fT*rkVelocity0;
    Vector2<Real> kMCenter = m_rkBox.Center + fT*rkVelocity1;
    Box2<Real> kMBox(kMCenter,m_rkBox.Axis,m_rkBox.Extent);
    return DistVector2Box2<Real>(kMVector,kMBox).GetSquared();
}
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// explicit instantiation
//----------------------------------------------------------------------------
template WM4_FOUNDATION_ITEM
class DistVector2Box2<float>;

template WM4_FOUNDATION_ITEM
class DistVector2Box2<double>;
//----------------------------------------------------------------------------
}
